''       S100 ConsoleIO board driver
''       John Monahan  (monahan@vitasoft.org)   5/25/2011
''

''      V28.0    5/25/2011      Corrected problem with DEL key


''      To be done:-
''      Need to find how to detect the keyboard scan code and display it in the first pair of HEX displays
''      Just noticed the CTRL key is not detected in keyboard spin module!. Need to be able to return CTRL characters.
    
CON
    _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000

    '' VGA driver: video output pin
    video = 16

    '' RS-232 driver values:  rx and tx lines
    r0 = 31
    t0 = 30

    '' Keyboard clock and data
    kbd = 14
    kbc = 15

    BS = 8
    LF = 10
    CR = 13
    ESC = 27
  
VAR

  byte  keyboard_char
  byte  keyboard_status
  byte  ch
 
OBJ
    te:         "VT100_Emulator-2"      ' ajvTerm code
    text:       "VGA_1024"              ' VGA Terminal Driver
    kb:         "Keyboard"              ' Keyboard driver
    ser0:       "FullDuplexSerial"      ' Full Duplex Serial Controller(s)



PUB main

    init_devices                                        ' One-time setup of Serial, VGA and Keyboard    

    te.init                                             ' initialize terinal emulator object
    te.start(video)                                     ' Start VGA output driver 
    
    dira[7..0]~~                                        'Set P7-P0 to output

    dira[8]~~                                           'To keep console IN bit "Busy" while processing data, always out
    outa[8] := 0                                        'nothing right now 

    dira[9]~                                            'DATA_IN_BUSY is read via this pin, alwauys in
    
    dira[10]~~                                          'Keyboard character available status bit, always out
    outa[10] := 0                                       'nothing right now

    dira[11]~                                           'DATA_OUT_BUSY is read via this pin, alwauys in

                                                        'P12 is unused
   
    dira[13]~~                                          'Pin for buzzer/bell - output
    outa[13] := 1
    
    dira[24]~~                                          'To latch U47
    outa[24] := 1                                       'Nothing initially
    dira[25]~~                                          'To latch U46
    outa[25] := 1                                       'Nothing initially

    dira[26]~~                                          'Set P26 to output for HEX display (keyboard key code)
    outa[26] := 1                                       'Nothing to latch right now
    dira[27]~~                                          'Set P27 to output for HEX display (keyboard ASCII translated key)
    outa[27] := 1                                       'no strobe or latch yet
                                                         
                                                        'P14 & P15 are for keyboard data & clock signals
 


    
    keyboard_status := 0                                'Flag as nothing there
    
    repeat

             if keyboard_status == 0                    'If nothing so far
                
                 keyboard_char := kb.key                'Check if something in que
                                                                                                                                    
                 if keyboard_char <> 0                  'non-zero means we have a character
                         
                         keyboard_status := $ff         'set flag we have a character

                         if keyboard_char == $c8        'Backspace keyboard key
                             keyboard_char := $08
                         if keyboard_char == $C9        'Delete Keyboard key (on Number Pad with NUM Lock Off) 
                             keyboard_char := $7F
                                                        '(Note you can play around here adding Characters or whole strings for the Function keys etc.)

                         if keyboard_char == $CB         'ESC key
                                keyboard_char := $1B

                         if keyboard_char == $D0         'cannot get any CTRL key to be recognized,  so for now F1 key = ^C
                                keyboard_char := $03
                                
                         if keyboard_char == $D1         'cannot get any CTRL key to be recognized,  so for now F2 key = ^P
                                keyboard_char := $10


                                
                         outa[7..0] := keyboard_char    'Update the HEX display
                         outa[27] := 0                  'Pulse line to latch data into HEX display
                         outa[27] := 1
                         
                         outa[24] := 0                  'Also latch data into U47
                         outa[24] := 1
                         
                         outa[10] := 1                  'Only then do we flag S-100 bus as something there (bit 1 of port 14H = 1)
                                                        'This is a little subtle. The strobe pulse from P24 will clock the 74LS74
                                                        'so StatusInBusy goes high. This will go to pin 2 of U39A. However only when
                                                        'P10 goes high on pin 1 of U39A does the status appear on the S100 bus indicating
                                                        'the board has a new keyboard character to be read. However the instant the S-100
                                                        'bus reads the data from U47 its pin 1 immediatly resets the 74LS74 removing the character available
                                                        'status bit from the S-100 bus.  The reason we have the "double" requirement for a
                                                        'status bit is because the p24 pulse (using Spin) is too long, causing the S-100 bus
                                                        'with a fast (eq 10Mhz Z80) to pick up two characters instead of one.
                                                         
             if keyboard_status == $ff                  'If we have a keyboard character
                                         
                 if ina[9] == %0                        'P9 Low, so character has now been read by S-100 bus (INPUT_ENABLE* went low resetting 7474, DATA_IN_BUSY)

                       outa[10] := 0                    'Flag S-100 bus no more characters available to be read
                       
                       keyboard_status := 0             'For next time     
                       
                      
          
            if ina[11] == %1                            'Is there an S-100 bus character WRITE request (P11, LATCHED_OUTPUT_ENABLE high)

                  outa[8] := 1                          'Raise Console Busy bit (P8 to pin 3 of LS02),
                                                        'this will lower set bit 2 of Console OUT Status port to 0

                  dira[7..0]~                           'Set P7-P0 to INPUT mode

                  outa[25] := 0                         'Lower 74LS373 OE pin (to place data on P0-P7 lines)
                                                        'Note this will also clear U44, LATCHED_OUTPUT_ENABLE, but because P8 is high/busy bit 2
                                                        'back to S-100 system is still low. This is necessary because the pulse signal from P25
                                                        'is long enough that a 10MHz Z80 can squeeze in another invalid console busy status bit check
                                                        
                  ch := ina[7..0]                       'Get the data
                  outa[25] := 1                         'Raise OE pine when done.   

                         
                      case ch
                                
                                $07:                                            'If Bell (07H) ring buzzer
                                        outa[13] :=  0
                                        waitcnt(clkfreq / 10 + cnt)             'wait for .1 seconds (100 ms)
                                        outa[13] := 1

                                $1A:                                            'For compatability with my SD Systems 8024 Video Board
                                   te.singleSerial0(27)                         '1AH clear Screen
                                   te.singleSerial0("[")                        'First home cursor
                                   te.singleSerial0("H")
                                   
                                   te.singleSerial0(27)                         'Then clear to EOS
                                   te.singleSerial0("[")                        
                                   te.singleSerial0("J")

                                $1C:                                             'For compatability with my SD Systems 8024 Video Board
                                   te.singleSerial0(27)                          '1CH clear from cursor position to EOL
                                   te.singleSerial0("[")                         
                                   te.singleSerial0("K")

                                $11:                                             'For compatability with my SD Systems 8024 Video Board
                                   te.singleSerial0(27)                          '11H turn off screen enhancements
                                   te.singleSerial0("[")                         
                                   te.singleSerial0("m")

                                $1E:                                             'For compatability with my SD Systems 8024 Video Board
                                   te.singleSerial0(27)                          '1eH home cursor
                                   te.singleSerial0("[")
                                   te.singleSerial0("H")
                                 
                                OTHER:
                                   te.singleSerial0(ch)                         'All other characters
                                         
                                                                          
                  dira[7..0]~~                                                  'Set P7-P0 back to OUTPUT
                        
                  outa[8] := 0                                                  'We are ready for next character
                   
                                                                             


'' Process bytes from our host port
PRI doSerial0 | c

    ' Consume bytes until FIFO is empty
    repeat
        c := ser0.rxcheck
        ser0.tx(c)

'' One-time initialization of terminal driver state
PRI init_devices

    ' Initialize RS-232 ports.  We'll shortly be restarting them
    '  after we choose a config
    ser0.start(r0, t0, 0, 9600)

    ' Start VGA output driver  <<<<<<<<<<< Inactivate simple VGA output for VT100 Mode >>>>>>>>>>>>
   ' text.start(video)
   ' text.cls

    ' Start Keyboard Driver
    kb.start(kbd, kbc)

'' Read and dispatch a keystroke


PRI doKey | key, ctl
    ' Get actual keystroke from driver
    key := kb.key
    if key == 0
      keyboard_char := 0
      keyboard_status := 0   
        return
        

    keyboard_char := key
    keyboard_status  := $ff

    ' Emit the character
    ser0.tx(key)
    text.putc(0,key)
    text.putc(90,key)
    


'' Write a string into the screen
PRI puts(row, col, str) | x, ptr
    ptr := (row * text#cols) + col
    repeat x from 0 to STRSIZE(str)-1
        text.putc(ptr++, BYTE[str+x])